##############################################################################
#
# Copyright (c) 2002 Ingeniweb SARL
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""
A document that can have an attachment
"""
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo, Role
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.ZAttachmentAttribute import ZAttachmentAttribute
from Products.CMFCore.utils import getToolByName
from DateTime import DateTime
from global_symbols import *

# ExFile definition for plone 1
factory_type_information = {
  'id'             : 'ExFile',
  'portal_type'    : 'ExFile',
  'meta_type'      : 'ExFile',
  'description'    : 'An ExFile is a content type with an Attachment.',
  'content_icon'   : 'ExFile.gif',
  'product'        : 'PloneExFile',
  'factory'        : 'addExFile',
  'immediate_view' : 'ExFile_editForm',
  'filter_content_types' : 0,
  'allowed_content_types': (),
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'ExFile_view',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'      : 'object',
    },
    {
    'id'           : 'edit',
    'name'         : 'Edit',
    'action'       : 'ExFile_editForm',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Edit Metadata',
    'action'        : 'metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    ),
  }

# ExFile definition for Plone 2
factory_type_information2 = {
  'id'             : 'ExFile',
  'portal_type'    : 'ExFile',
  'meta_type'      : 'ExFile',
  'description'    : 'An ExFile is a content type with an Attachment.',
  'content_icon'   : 'ExFile.gif',
  'product'        : 'PloneExFile',
  'factory'        : 'addExFile',
  'immediate_view' : 'ExFile_editForm2',
  'filter_content_types' : 0,
  'allowed_content_types': (),
  'actions': (
    {
    'id'           : 'view',
    'name'         : 'View',
    'action'       : 'string:${object_url}/ExFile_view2',
    'permissions'  : (CMFCorePermissions.View, ),
    'category'     : 'object',
    },
    {
    'id'            : 'edit',
    'name'          : 'Edit',
    'action'        : 'string:${object_url}/ExFile_editForm2',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'lock_unlock',
    'name'          : 'Unlock (Currently locked)',
    'action'        : 'string:${object_url}/unlockExFile',
    'permissions'   : (CMFCorePermissions.View, ),
    'condition'     : 'python:object.canUnlock()',
    'category'      : 'document_actions',
    },
    {
    'id'            : 'unlock_lock',
    'name'          : 'Lock (Currently not locked)',
    'action'        : 'string:${object_url}/lockExFile',
    'permissions'   : (CMFCorePermissions.View, ),
    'condition'     : 'python:object.canLock()',
    'category'      : 'document_actions',
    },
    {
    'id'            : 'metadata',
    'name'          : 'Properties',
    'action'        : 'string:${object_url}/metadata_edit_form',
    'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
    'category'      : 'object',
    },
    {
    'id'            : 'sharing',
    'name'          : 'Sharing',
    'action'        : 'string:${object_url}/folder_localrole_form',
    'permissions'   : (CMFCorePermissions.ManageProperties, ),
    'category'      : 'object',
    },
    {
    'id'            : 'content_status_history',
    'name'          : 'State',
    'action'        : 'string:${object_url}/content_status_history',
    'condition'     : 'python:object and portal.portal_workflow.getTransitionsFor(object, object.getParentNode())',
    'permissions'   : (CMFCorePermissions.View,),
    'category'      : 'object_tabs',
    'visible'       : 0
    },
    ),
  }

# The usual factory
def addExFile(self, id, title = '', REQUEST = {}):
    """
    Factory method for a ExFile object
    """
    obj = ExFile(id, title)
    self._setObject(id, obj)

    if REQUEST.has_key('RESPONSE'):
        return REQUEST.RESPONSE.redirect('manage_main')



class ExFile(PortalContent, DefaultDublinCoreImpl, Role.RoleManager):
    """ExFile class"""

    #                                                                           #
    #                            BASIC CLASS BEHAVIOUR                          #
    #                                                                           #

    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )
    meta_type = 'ExFile'
    _isDiscussable = 1
    manage_options = PortalContent.manage_options

    # Standard security settings
    security = ClassSecurityInfo()
    # The following method makes all class properties & methods protected by View by default
    security.declareObjectProtected(CMFCorePermissions.View)

    def __init__(self, id, title='', description='', ):
        DefaultDublinCoreImpl.__init__(self)
        self.id = id
        self.title = title
        self.description = description
        self.attach = ZAttachmentAttribute.ZAttachmentAttribute(id = "attach")
        self._exfile_locked=0
        self._exfile_locked_by=None
        self._exfile_locked_date=None
        self._exfile_unlocked_by=None
        self._exfile_unlocked_date=None

    def _post_init(self):
        """
        _post_init(self) => Post-init method (that is, method that is called AFTER the class has been set into the ZODB)
        """
        self.indexObject()

    security.declarePublic('getIcon')
    def getIcon(self, relative_to_portal = 0):
        """
        getIcon(self, relative_to_portal = 0) => Return content-type-based icon
        """
        # Find the proper icon (using ZAA if necessary)
        try:
            m = getattr(self.attach, 'getSmallIcon', self.attach.getIcon)       # This preserves ZAA 1.0 compatibility, with
            icon = m()                                                          # a big icon instead of the small one
        except:
            icon = "ExFile_small.gif"

        # Return it
        if relative_to_portal:
            return icon
        else:
            # Relative to REQUEST['BASEPATH1']
            portal_url = getToolByName( self, 'portal_url' )
            res = portal_url(relative=1) + '/' + icon
            while res[:1] == '/':
                res = res[1:]
            return res



    #                                                                           #
    #                           CONTENT EDITING METHODS                         #
    #                                                                           #



    # Edit method (change this to suit your needs)
    # This edit method should only change attributes that are neither 'id' nor metadata.
    security.declareProtected(ExFile_editPermission, 'edit')
    def edit(self, title = '', REQUEST = {}):
        """
        edit(self, title = '', REQUEST = {}) => object modification method
        """
        # Change attributes
        self.title = title

        # Reindex
        self.reindexObject()


    security.declareProtected(ExFile_editPermission, 'deleteFile')
    def deleteFile(self, REQUEST = {}):
        """
        deleteFile(self, REQUEST = {}) => remove the attachment
        """
        # Delete file
        self.attach.delete()
        self.attach = ZAttachmentAttribute.ZAttachmentAttribute()

        # Reindex catalog
        self.reindexObject()


    security.declareProtected(ExFile_editPermission, 'attachFile')
    def attachFile(self, title = '', file = '', REQUEST = {}):
        """
        attachFile(self, title = '', file = '', REQUEST = {}) => file attaching method

        If no file is given, or the file is empty, the original file is preserved.
        If you want to delete the file, use deleteFile instead.
        """
        if file:
            self.attach.upload(file)
        self.attach.setTitle(title)

        # Enforce persistancy
        self.attach = self.attach

        # Reindex catalog
        self.reindexObject()

    security.declareProtected(CMFCorePermissions.View, 'hasAttachment')
    def hasAttachment(self):
        return not self.attach.isEmpty()
    
    #this method is used by folder_contents to show size
    security.declareProtected(CMFCorePermissions.View, 'get_size')
    def get_size(self,):
        if self.hasAttachment():
            return self.attach.getSize()
        return 0

    #                                                                           #
    #                             LOCKING SUPPORT                               #
    #                                                                           #

    security.declareProtected(ExFile_editPermission, 'unlock')
    def lock(self):
        """
        lock(self) - Locks the ExFile
        """
        if not self.isLocked() and self.hasAttachment():
            now = DateTime()
            portal_membership = getToolByName(self, 'portal_membership')
            username = portal_membership.getAuthenticatedMember().getUserName()
            self._exfile_locked=1
            self._exfile_locked_by=username
            self._exfile_locked_date=now

    security.declareProtected(ExFile_editPermission, 'lock')
    def unlock(self):
        """
        unlock(self) - Unlocks the ExFile if it is locked and the user is the
                       same that locked the ExFile or tries to force the unlock.
        """
        portal_membership = getToolByName(self, 'portal_membership')
        if self.isLocked():
            username = portal_membership.getAuthenticatedMember().getUserName()
            if username == self._exfile_locked_by:
                now = DateTime()
                self._exfile_locked=0
                self._exfile_unlocked_by=username
                self._exfile_unlocked_date=now
            else:
                # Let's try to force the unlock
                self.force_unlock()

    security.declareProtected(ExFile_forceUnlockPermission, 'force_unlock')
    def force_unlock(self):
        """
        Forces an unlock if the user is allowed to.
        """
        if self.isLocked():
            now = DateTime()
            portal_membership = getToolByName(self, 'portal_membership')
            member=portal_membership.getAuthenticatedMember()
            username = member.getUserName()
            if portal_membership.checkPermission(ExFile_forceUnlockPermission, self):
                self._exfile_locked=0
                self._exfile_unlocked_by=username
                self._exfile_unlocked_date=now

    security.declareProtected(CMFCorePermissions.View, 'isLocked')
    def isLocked(self):
        """Returns 1 if ExFile is locked otherwise 0"""
        locked = getattr(self, '_exfile_locked', 0)
        return locked

    security.declareProtected(CMFCorePermissions.View, 'LockedBy')
    def LockedBy(self):
        """Returns the username of the member which has locked the document"""
        locked_by = getattr(self, '_exfile_locked_by', None)
        return locked_by

    security.declareProtected(CMFCorePermissions.View, 'LockedDate')
    def LockedDate(self):
        """Returns the date when the ExFile was locked"""
        locked_date = getattr(self, '_exfile_locked_date', None)
        return locked_date

    security.declareProtected(CMFCorePermissions.View, 'canLock')
    def canLock(self):
        """Convenience method that is used to decide whether the lock action
           icon is shown.
           The Lock icon is shown if the ExFile is not locked, has an Attachment and the 
           member has the permission to edit the ExFile.
        """
        portal_membership = getToolByName(self, 'portal_membership')
        member=portal_membership.getAuthenticatedMember()
        can_lock=0
        if not self.isLocked() and self.hasAttachment():
            if portal_membership.checkPermission(ExFile_editPermission, self):
                can_lock=1
        return can_lock

    security.declareProtected(CMFCorePermissions.View, 'canUnlock')
    def canUnlock(self):
        """Convenience method that is used to decide whether the unlock action
           icon is shown.
           The Unlock icon is shown if the ExFile is locked and the member has locked the
           ExFile and still has the permission to edit it.
           The Unlock icon is also shown if the ExFile is locked and the member has the
           permission to force an unlock.
        """
        portal_membership = getToolByName(self, 'portal_membership')
        member=portal_membership.getAuthenticatedMember()
        can_unlock=0
        if self.isLocked():
            if portal_membership.checkPermission(ExFile_editPermission, self):
                if self._exfile_locked_by==member.getUserName():
                    can_unlock=1
                elif portal_membership.checkPermission(ExFile_forceUnlockPermission, self):
                    can_unlock=1
        return can_unlock

    security.declareProtected(CMFCorePermissions.View, 'canWork')
    def canWork(self):
        """Convenience method that is used to decide whether the edit form is shown.

           If the ExFile is locked, it can only be edited by the a person that
           can unlock it.
           If the ExFile is not locked, the edit permission is sufficient.
        """
        portal_membership = getToolByName(self, 'portal_membership')
        member=portal_membership.getAuthenticatedMember()
        can_work=0
        if self.isLocked() and self.canUnlock():
            can_work=1
        elif not self.isLocked() and portal_membership.checkPermission(ExFile_editPermission, self):
            can_work=1
        return can_work

    #                                                                           #
    #                             CMF CATALOG SUPPORT                           #
    #                                                                           #

    security.declareProtected(CMFCorePermissions.View, 'SearchableText')
    def SearchableText(self):
        """Returns a concatination of all searchable text"""
        # Should be overriden by portal objects
        return "%s %s %s" % (self.Title(), self.Description(), self.attach.SearchableText())

# Class instanciation
InitializeClass(ExFile)


