#
# PloneExFile tests
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))


from Products.PloneTestCase import PloneTestCase
PloneTestCase.installProduct('PloneExFile')

PloneTestCase.setupPloneSite()
from Testing import ZopeTestCase

from Products.PloneExFile.global_symbols import *
from Acquisition import aq_base
from AccessControl import Unauthorized

try:
    import Log

    Log.LOG_LEVEL = Log.LOG_DEBUG

    Log.LOG_PROCESSOR = {
        Log.LOG_NONE: Log.logFile,
        Log.LOG_CRITICAL: Log.logFile,
        Log.LOG_ERROR: Log.logFile,
        Log.LOG_WARNING: Log.logFile,
        Log.LOG_NOTICE: Log.logFile,
        Log.LOG_DEBUG: Log.logFile,
        }
    from Log import *
    Log(LOG_NOTICE, "Starting %s at %d debug level" % (os.path.dirname(__file__), LOG_LEVEL, ))
except:
    print "Log module not available"
    LOG_DEBUG = None
    LOG_NOTICE = None
    LOG_WARNING = None
    LOG_ERROR = None
    LOG_CRITICAL = None
    def Log(*args, **kw):
        pass
    raise


# Fake upload object
class TestFile:
    __allow_access_to_unprotected_subobjects__ = 1
    filename = 'test.txt'
    headers ={'Content-Type':'text/plain'}
    def seek(*args): pass
    def tell(*args): return 0
    def read(*args): return 'file_contents'


class TestPloneExFile(PloneTestCase.PloneTestCase):
    """ Basic test class, should be more complete """
    
    def createMember(self, id, pw, email, roles=('Member',)):
        pr = self.portal.portal_registration
        member = pr.addMember(id, pw, roles, properties={ 'username': id, 'email' : email })
        return member
    
    def installation(self, productName):
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct(productName)
        
    def afterSetUp(self, ):
        self.installation('PloneExFile')
        
        self.catalog = self.portal.portal_catalog
        self.workflow = self.portal.portal_workflow
        self.membership = self.portal.portal_membership
        
        self.contentId = 'myFile'
        self.folder.invokeFactory(type_name='ExFile', id=self.contentId)
        self.failUnless(self.catalog(id=self.contentId))
        self.exfile = getattr(self.folder, self.contentId)
        
        #Members
        self.user2 = self.createMember('user2', 'abcd4', 'abc@domain.tld')
        
    
    def testAddPloneExFile(self):
        """ test if PloneExFile is created """
        self.failUnless(hasattr(aq_base(self.folder), self.contentId))

    def testIsIndexed(self):
        """ test if PloneExFile is indexed """
        self.failUnless(self.catalog(id=self.contentId))

    def testReindexPloneExFile(self):
        """ Modify the title and re index PloneExFile """
        self.failIf(self.catalog(id=self.contentId, Title='My File'))
        self.exfile.setTitle('My File')
        self.exfile.reindexObject()
        self.failUnless(self.catalog(id=self.contentId, Title='My File'))
    
    #Standard Plone tools
    
    def testDeletePloneExFile(self,):
        """ delete PloneExFile """
        self.folder._delObject(self.contentId)
        self.failIf(self.catalog(id=self.contentId))
        Log(LOG_NOTICE, "PloneExFile deleted.")
    
    
    
    ###############################
    #PloneExFile related method#
    ###############################
    
    def testEditPloneExFile(self,):
        """ edit the PloneExFile """
        title = 'My File'
        description = 'Document speaking of general economy'
        self.exfile.setTitle(title)
        self.exfile.setDescription(description)
        
        #edit our product with title and description
        self.assertEqual(self.exfile.Title(), title)
        self.assertEqual(self.exfile.Description(), description)
        Log(LOG_NOTICE, "PloneExFile edited.")
    
    def testAddFile(self,):
        """ 
        Add a file, 
        verify hasAttachment(), 
        verify if file is correctly uploaded.
        """
        self.exfile.attachFile(title = 'my file', file = TestFile())
        self.assertEqual(self.exfile.hasAttachment(), 1)
        Log(LOG_DEBUG, str(self.exfile.attach.getFile()))
        self.assertEqual(str(self.exfile.attach.getFile()), 'file_contents')
    
    def testDeleteFile(self,):
        """ Add a file and delete it """
        self.exfile.attachFile(title = 'my file', file = TestFile())
        self.assertEqual(self.exfile.hasAttachment(), 1)
        self.exfile.deleteFile()
        self.assertEqual(self.exfile.hasAttachment(), 0)
    
    ######
    #LOCK#
    ######
    def testLockPloneExFile(self,):
        """ lock an exfile """
        #must have a file in it, so user can lock it
        self.exfile.attachFile(title = 'my file', file = TestFile())
        #self.assertEqual(self.exfile.canLock(), 1)
        self.assertEqual(self.exfile.canWork(), 1)
        self.exfile.lock()
        #self.assertEqual(self.exfile.canUnlock(), 1)
        self.assertEqual(self.exfile.canWork(), 1)
        Log(LOG_NOTICE, "lock by " + str(self.exfile.LockedBy()))
        Log(LOG_NOTICE, "lock date " + str(self.exfile.LockedDate()))
        self.assertEqual(self.exfile.isLocked(), 1)
    
    def testUnlockExFile(self,):
        """ lock an exfile """
        self.testLockPloneExFile()
        self.exfile.unlock()
        self.assertEqual(self.exfile.canWork(), 1)
        self.assertEqual(self.exfile.isLocked(), 0)
        
    
    def testAccessAfterLock(self, ):
        """
        user1 locks an exfile and logout
        user2 has login, he must not be able to edit
        """
        self.testLockPloneExFile()
        self.logout()
        self.login('user2')
        self.assertEqual(self.membership.getAuthenticatedMember().getUserName(), 'user2')
        self.assertEqual(self.exfile.canWork(), 0)
        
        def testAnotherUserLock(self, ):
        """
        user1 locks an exfile and logout
        user2 has login and locks the exfile
        user1 must not be able to edit the exfile
        """
        #must have a file in it, so user can lock it
        self.exfile.attachFile(title = 'my file', file = TestFile())
        self.logout()
        
        self.login('user2')
        user2Name = self.membership.getAuthenticatedMember().getUserName()
        self.assertEqual(user2Name, 'user2')
        #normally : user2 cannot edit
        self.assertEqual(self.exfile.canWork(), 0)
        #give permission to edit
        self.setPermissions(ExFile_editPermission)
        self.assertEqual(self.exfile.canLock(), 1)
        self.assertEqual(self.exfile.canWork(), 1)
        
        self.exfile.lock()
        self.assertEqual(self.exfile.isLocked(), 1)
        self.assertEqual(str(self.exfile.LockedBy()), user2Name)
        
        self.logout()
        self.login(ZopeTestCase.user_name) #test_user_1_
        self.assertEqual(self.exfile.canWork(), 0)
    
    def testForceUnlockExFile(self,):
        """
        user1 locks an exfile and logout
        user2 has login, has the ExFile_forceUnlockPermission and force unlock.
        """
        self.testLockPloneExFile()
        self.logout()
        self.login('user2')
        self.assertEqual(self.membership.getAuthenticatedMember().getUserName(), 'user2')
        self.assertEqual(self.exfile.canWork(), 0)
        #self.assertEqual(self.exfile.canUnlock(), 0)
        #give the user the permission to force unlock
        self.setPermissions(ExFile_forceUnlockPermission)
        self.exfile.force_unlock()
        self.assertEqual(self.exfile.isLocked(), 0)
        
        
    
if __name__ == '__main__':
    framework()
else:
    def test_suite():
        from unittest import TestSuite, makeSuite
        suite = TestSuite()
        suite.addTest(makeSuite(TestPloneExFile))
        return suite

